// code for reading from MLX90614 derived from bildr community source
//      http://bildr.org/2011/02/mlx90614-arduino/

#include "Arduino.h"
#include "i2cmaster.h"
#include "osv3_dock_sensor.h"
#include "mlx90614_IR_temperature.h"


Mlx90614IrTemp::Mlx90614IrTemp()
  : Osv3Thgr810Sensor(TEMP_CHANNEL, TX_DATA_PIN, TX_POWER_PIN),
    m_temperature(15)     // 15 minutes of data to average
{
  i2c_init(); // Initialise the i2c bus
  PORTC = (1 << PORTC4) | (1 << PORTC5); // enable pullups
  
  // configure pins
  pinMode(TEMP_POWER_PIN, OUTPUT);

  digitalWrite(TEMP_POWER_PIN, LOW);
}

Mlx90614IrTemp::~Mlx90614IrTemp()
{
}


void Mlx90614IrTemp::loop(void)
{
  // called once per second
  static int callCount = m_secondsBetweenTransmits[TEMP_CHANNEL -1] - 3;  // want to send packet out immediately on start up
  float temp;
  float temp2;
  
  callCount++;
  
  // turn power on 3 seconds before reading
  if (callCount > m_secondsBetweenTransmits[TEMP_CHANNEL - 1] - 3)
  {
    digitalWrite(TEMP_POWER_PIN, HIGH);
  }

  // only want to process temp when its time to transmit
  if (callCount < m_secondsBetweenTransmits[TEMP_CHANNEL - 1])
  {
    return;
  }

  callCount = 0;
  
  digitalWrite(LED_PIN, HIGH);
  
  
  // read twice to error check data
  temp = readTemp();
  delay(100);
  
  temp2 = readTemp();
  
  // allow 1 degree variation max
  if ((abs)(temp - temp2) > 1.0)
  {
    Serial.print("bad readings: "); 
    Serial.print(temp); 
    Serial.print(" vs "); 
    Serial.println(temp2); 
    return;    
  }
  
  m_temperature.store((temp + temp2) / 2.0);

  buildAndSendTempPacket();
  
  digitalWrite(TEMP_POWER_PIN, LOW);
  digitalWrite(LED_PIN, LOW);  
}

float Mlx90614IrTemp::readTemp(void)
{
  // code for reading from MLX90614 derived from bildr community source
  int dev = 0x5A << 1;
  int data_low = 0;
  int data_high = 0;
  int pec = 0;
  
  i2c_start_wait(dev + I2C_WRITE);
  i2c_write(0x07);
  
  // read
  i2c_rep_start(dev + I2C_READ);
  data_low = i2c_readAck(); //Read 1 byte and then send ack
  data_high = i2c_readAck(); //Read 1 byte and then send ack
  pec = i2c_readNak();
  i2c_stop();
  
  //This converts high and low bytes together and processes temperature, MSB is a error bit and is ignored for temps
  double tempFactor = 0.02; // 0.02 degrees per LSB (measurement resolution of the MLX90614)
  double tempData = 0x0000; // zero out the data
  int frac; // data past the decimal point
  
  // This masks off the error bit of the high byte, then moves it left 8 bits and adds the low byte.
  tempData = (double)(((data_high & 0x007F) << 8) + data_low);
  tempData = (tempData * tempFactor)-0.01;
  
  float celcius = tempData - 273.15;
  float fahrenheit = (celcius * 1.8) + 32;
  
  Serial.print("Celcius: ");
  Serial.println(celcius);
  
  Serial.print("Fahrenheit: ");
  Serial.println(fahrenheit);

  return celcius;  
}

void Mlx90614IrTemp::buildAndSendTempPacket(void)
{
  float temp = m_temperature.average();
  
  Serial.println("Send packet");
  Serial.print("temp: ");
  Serial.println(temp);

  // humidity field is not used for any specific data in this packet

  // send high value
  buildAndSendPacket(temp, 90);
}
