// information for this sensor was gleened from multiple places
// - the "Oregon Scientific RF Protocol Description" from May 2011 has data packet format information
//      http://wmrx00.sourceforge.net/Arduino/OregonScientific-RF-Protocols.pdf

// - sourceforge.net forums discussion of the CRC in the data packet

// - crc calculation code was taken from crc8.c published by Rajiv Chakravorty
//      http://www.rajivchakravorty.com/source-code/uncertainty/multimedia-sim/html/crc8_8c-source.html

// - code for taking to the ultrasonic distance measurement derived from the UltrasoundSensor tutorial
//      http://arduino.cc/en/Tutorial/UltrasoundSensor

// - code for reading from MLX90614 derived from bildr community source
//      http://bildr.org/2011/02/mlx90614-arduino/

// parts used include
// - Arduino Uno
// - MLX90614 IR temp sensor from digikey
// - HC-SR04 ultrasonic ping sensor from ebay
// - generic 4-pin 433MHz transmitter; e.g. http://www.rentron.com/remote_control/TWS-434.htm



#include "osv3_dock_sensor.h"
#include "hc_sr04_ping.h"
#include "mlx90614_IR_temperature.h"
#include "i2cmaster.h"     // if this library include file is not included here, then the cpp file cannot find it


// sensors
HcSr04Ping waterLevelSensor;
Mlx90614IrTemp temperatureSensor;


void setup() 
{
  Serial.begin(57600);
  Serial.println("Oregon Scientific Protocol v3.0 Compatible Water Hieght and Temp Transmitter v 0.92");

  // configure pins
  pinMode(LED_PIN, OUTPUT);
  
  // allow powered up items time to ready themselves
  delay(1000);
}

void loop()
{
  static unsigned long startTime = millis();
  int sleepTime;

  // service sensors once per second
  // sensors will handle their own timing needs beyond that  

  temperatureSensor.loop();
  waterLevelSensor.loop();
  
  // 1 second between loops
  // all sensors combined processing time needs to be < 1 second
  sleepTime = 1000 - (millis() - startTime);
  if (sleepTime > 0)
  {
    delay(sleepTime);
  }
  else
  {
    Serial.print("Slow loop, no sleep: ");
    Serial.println(sleepTime);
  }
  
  startTime += 1000;
}
